@@ -8,12 +8,12 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *                                                                              
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *                                                                              
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -24,16 +24,15 @@
 #include <config.h>
 #endif
 
+#include "xfae-window.h"
 #include "xfae-dialog.h"
 #include "xfae-model.h"
-#include "xfae-window.h"
 
+#include <libxfce4ui/libxfce4ui.h>
 
-
-static void     xfae_window_class_init          (XfaeWindowClass  *klass);
-static void     xfae_window_init                (XfaeWindow       *window);
 static void     xfae_window_add                 (XfaeWindow       *window);
 static void     xfae_window_remove              (XfaeWindow       *window);
+static void     xfae_window_edit                (XfaeWindow       *window);
 static gboolean xfae_window_button_press_event  (GtkWidget        *treeview,
                                                  GdkEventButton   *event,
                                                  XfaeWindow       *window);
@@ -179,11 +178,22 @@ xfae_window_init (XfaeWindow *window)
   button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
   g_signal_connect_swapped (G_OBJECT (button), "clicked",
                             G_CALLBACK (xfae_window_remove), window);
+  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
   g_signal_connect (G_OBJECT (window->selection), "changed",
                     G_CALLBACK (xfae_window_selection_changed), button);
   xfae_window_selection_changed (window->selection, button);
+
+  button = gtk_button_new_from_stock (GTK_STOCK_EDIT);
+  g_signal_connect_swapped (G_OBJECT (button), "clicked",
+                            G_CALLBACK (xfae_window_edit), window);
   gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
+
+  g_signal_connect (G_OBJECT (window->selection), "changed",
+                    G_CALLBACK (xfae_window_selection_changed), button);
+  xfae_window_selection_changed (window->selection, button);
 }
 
 
@@ -257,14 +267,16 @@ static void
 xfae_window_add (XfaeWindow *window)
 {
   GtkTreeModel *model;
+  GtkWidget    *parent;
   GtkWidget    *dialog;
   GError       *error = NULL;
   gchar        *name;
   gchar        *descr;
   gchar        *command;
 
-  dialog = xfae_dialog_new ();
-  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (window));
+  dialog = xfae_dialog_new (NULL, NULL, NULL);
+  parent = gtk_widget_get_toplevel (GTK_WIDGET (window));
+  gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
   if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
     {
       gtk_widget_hide (dialog);
@@ -274,7 +286,7 @@ xfae_window_add (XfaeWindow *window)
       model = gtk_tree_view_get_model (GTK_TREE_VIEW (window->treeview));
       if (!xfae_model_add (XFAE_MODEL (model), name, descr, command, &error))
         {
-          xfce_err (error->message);
+          xfce_dialog_show_error (GTK_WINDOW (parent), error, _("Failed adding \"%s\""), name);
           g_error_free (error);
         }
 
@@ -294,13 +306,32 @@ xfae_window_remove (XfaeWindow *window)
   GtkTreeModel     *model;
   GtkTreeIter       iter;
   GError           *error = NULL;
+  GtkWidget        *parent;
+  gchar            *name;
+  gboolean          remove_item;
+
+  parent = gtk_widget_get_toplevel (GTK_WIDGET (window));
 
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->treeview));
   if (gtk_tree_selection_get_selected (selection, &model, &iter))
     {
-      if (!xfae_model_remove (XFAE_MODEL (model), &iter, &error))
+      if (!xfae_model_get (XFAE_MODEL (model), &iter, &name, NULL, NULL, &error))
         {
-          xfce_err (error->message);
+          xfce_dialog_show_error (GTK_WINDOW (parent), error, _("Failed to remove item"));
+          g_error_free (error);
+          return;
+        }
+
+      remove_item = xfce_dialog_confirm (GTK_WINDOW (parent), GTK_STOCK_REMOVE, NULL,
+                                         _("This will permanently remove the application "
+                                           "from the list of automatically started applications"),
+                                         _("Are you sure you want to remove \"%s\""), name);
+
+      g_free (name);
+
+      if (remove_item && !xfae_model_remove (XFAE_MODEL (model), &iter, &error))
+        {
+          xfce_dialog_show_error (GTK_WINDOW (parent), error, _("Failed to remove item"));
           g_error_free (error);
         }
     }
@@ -308,6 +339,59 @@ xfae_window_remove (XfaeWindow *window)
 
 
 
+static void
+xfae_window_edit (XfaeWindow *window)
+{
+  GtkTreeSelection *selection;
+  GtkTreeModel     *model;
+  GtkTreeIter       iter;
+  GError           *error = NULL;
+  gchar            *name;
+  gchar            *descr;
+  gchar            *command;
+  GtkWidget        *parent;
+  GtkWidget        *dialog;
+
+  parent = gtk_widget_get_toplevel (GTK_WIDGET (window));
+
+  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (window->treeview));
+  if (gtk_tree_selection_get_selected (selection, &model, &iter))
+    {
+      if (!xfae_model_get (XFAE_MODEL (model), &iter, &name, &descr, &command, &error))
+        {
+          xfce_dialog_show_error (GTK_WINDOW (parent), error, _("Failed to edit item"));
+          g_error_free (error);
+          return;
+        }
+
+      dialog = xfae_dialog_new (name, descr, command);
+      gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent));
+
+      g_free (command);
+      g_free (descr);
+      g_free (name);
+
+      if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_OK)
+        {
+	  gtk_widget_hide (dialog);
+
+	  xfae_dialog_get (XFAE_DIALOG (dialog), &name, &descr, &command);
+
+          if (!xfae_model_edit (XFAE_MODEL (model), &iter, name, descr, command, &error))
+            {
+              xfce_dialog_show_error (GTK_WINDOW (parent), error, _("Failed to edit item \"%s\""), name);
+              g_error_free (error);
+            }
+
+          g_free (command);
+          g_free (descr);
+          g_free (name);
+        }
+      gtk_widget_destroy (dialog);
+    }
+}
+
+
 static void
 xfae_window_item_toggled (XfaeWindow *window,
                           gchar      *path_string)
@@ -323,7 +407,7 @@ xfae_window_item_toggled (XfaeWindow *window,
     {
       if (!xfae_model_toggle (XFAE_MODEL (model), &iter, &error))
         {
-          xfce_err (error->message);
+          xfce_dialog_show_error (NULL, error, _("Failed to toggle item"));
           g_error_free (error);
         }
     }
@@ -368,11 +452,11 @@ xfae_window_new (void)
 /**
  * xfae_window_create_plug_child:
  *
- * Creates a widget that can be used to embed the window contents 
+ * Creates a widget that can be used to embed the window contents
  * into a GtkPlug widget. After this function call, the XfaeWindow can
  * no longer be used and has to be re-created.
  *
- * Return value: A widget holding the most important contents of the 
+ * Return value: A widget holding the most important contents of the
  *               window.
  **/
 GtkWidget*
@@ -406,12 +490,18 @@ xfae_window_create_plug_child (XfaeWindow *window)
   button = gtk_button_new_from_stock (GTK_STOCK_REMOVE);
   g_signal_connect_swapped (G_OBJECT (button), "clicked",
                             G_CALLBACK (xfae_window_remove), window);
-  g_signal_connect (G_OBJECT (window->selection), "changed",
-                    G_CALLBACK (xfae_window_selection_changed), button);
-  xfae_window_selection_changed (window->selection, button);
   gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
   gtk_widget_show (button);
 
+  button = gtk_button_new_from_stock (GTK_STOCK_EDIT);
+  g_signal_connect_swapped (G_OBJECT (button), "clicked",
+                            G_CALLBACK (xfae_window_edit), window);
+  gtk_box_pack_start (GTK_BOX (bbox), button, FALSE, FALSE, 0);
+  gtk_widget_show (button);
+
+  g_signal_connect (G_OBJECT (window->selection), "changed",
+                    G_CALLBACK (xfae_window_selection_changed), button);
+  xfae_window_selection_changed (window->selection, button);
   return vbox;
 }
 #endif
