@@ -1,4 +1,4 @@
-/* $Id: xfsm-client.h 20833 2006-04-09 18:08:17Z benny $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2003-2004 Benedikt Meurer <benny@xfce.org>
  * All rights reserved.
@@ -7,12 +7,12 @@
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2, or (at your option)
  * any later version.
- *                                                                              
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *                                                                              
+ *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -22,8 +22,19 @@
 #ifndef __XFSM_CLIENT_H__
 #define __XFSM_CLIENT_H__
 
+#include <glib-object.h>
+
 #include <xfce4-session/xfsm-properties.h>
 
+G_BEGIN_DECLS
+
+#define XFSM_TYPE_CLIENT     (xfsm_client_get_type ())
+#define XFSM_CLIENT(obj)     (G_TYPE_CHECK_INSTANCE_CAST((obj), XFSM_TYPE_CLIENT, XfsmClient))
+#define XFSM_IS_CLIENT(obj)  (G_TYPE_CHECK_INSTANCE_TYPE((obj), XFSM_TYPE_CLIENT))
+
+/* fwd decl */
+struct _XfsmManager;
+
 typedef struct _XfsmClient XfsmClient;
 
 typedef enum
@@ -37,26 +48,35 @@ typedef enum
   XFSM_CLIENT_WAITFORPHASE2,
   XFSM_CLIENT_DISCONNECTED,
 } XfsmClientState;
-  
 
-struct _XfsmClient
-{
-  XfsmClientState state;
-  const gchar    *id;
-  XfsmProperties *properties;
-  SmsConn sms_conn;
-  guint save_timeout_id;
-};
+GType xfsm_client_get_type (void) G_GNUC_CONST;
 
-
-#define XFSM_CLIENT(c) ((XfsmClient *) (c))
-
-
-XfsmClient *xfsm_client_new (SmsConn sms_conn);
-
-void xfsm_client_free (XfsmClient *client);
+XfsmClient *xfsm_client_new (struct _XfsmManager *manager,
+                             SmsConn              sms_conn);
 
 void xfsm_client_set_initial_properties (XfsmClient     *client,
                                          XfsmProperties *properties);
 
+XfsmClientState xfsm_client_get_state (XfsmClient *client);
+void xfsm_client_set_state (XfsmClient     *client,
+                            XfsmClientState state);
+
+G_CONST_RETURN gchar *xfsm_client_get_id (XfsmClient *client);
+
+SmsConn xfsm_client_get_sms_connection (XfsmClient *client);
+
+XfsmProperties *xfsm_client_get_properties (XfsmClient *client);
+XfsmProperties *xfsm_client_steal_properties (XfsmClient *client);
+
+void xfsm_client_merge_properties (XfsmClient *client,
+                                   SmProp    **props,
+                                   gint        num_props);
+void xfsm_client_delete_properties (XfsmClient *client,
+                                    gchar     **prop_names,
+                                    gint        num_props);
+
+G_CONST_RETURN gchar *xfsm_client_get_object_path (XfsmClient *client);
+
+G_END_DECLS
+
 #endif /* !__XFSM_CLIENT_H__ */
